import p3dFilt as p3dFilt
from p3dFilt import *

################## 

def py_printErrorMessage(err_code):
  
  string = "Success. \n"
  
  if err_code == 1 or err_code == 0:
  	string = "Error on code execution. \n"
  	return
  	
  if err_code == -2:
   	string = "Input argument IMAGE must be of type BYTE or UINT. \n"
   	return
  	
  if err_code == -3:
   	string = "Input argument IMAGE must be a 2D or 3D matrix. \n"
   	return
   	
  if err_code == 2:
  	print (string)


################## Handle 8 bit 

#read
def py_p3dReadRaw8(filename, dimx,dimy,dimz):
	"""
    This function reads a RAW image in 8-bit format from the specified file.
 
 Syntax:
 ------
 Result = py_p3dReadRaw8 ( FileName, dimx, dimy, dimz)
 
 Return Value:
 ------------
 Returns a matrix of type BYTE with the dimensions specified in input representing the image read from disk.
 
 Arguments:
 ---------
 Filename: A string with the filename with full path included.
 dimx,dimy,dimz: three variables representing the dimensions of image to read. 
 
 Remarks:
 -------
 Trying to read a RAW image stored in SIGNED format is in principle an error. 
 Also, #using p3dReadRaw8 for a 16-bit RAW image results in a completely wrong output but, again, no error could be detected.
 Attention #must be paid when using this function. Pore3D internally deals only with UNSIGNED format.
	"""
	image_data = malloc_uchar(dimx*dimy*dimz)
	err_code = p3dReadRaw8(filename, image_data,dimx,dimy,dimz, None, None);
	py_printErrorMessage(err_code)
	return image_data

#write
def py_p3dWriteRaw8(image_data, filename, dimx,dimy,dimz):
	#image_data = p3dFilt.malloc_uchar(dimx*dimy*dimz)
	err_code = p3dWriteRaw8(image_data, filename,dimx,dimy,dimz, None, None);
	py_printErrorMessage(err_code)
	return image_data

#Gaussian Filter	
def py_p3dGaussianFilter8(image_data, dimx,dimy,dimz, width =3 ,sigma = 1.0):
	out_image = p3dFilt.malloc_uchar(dimx*dimy*dimz)
	err_code = p3dGaussianFilter3D_8(image_data,out_image,dimx,dimy,dimz, 3,1.0, None, None)
	py_printErrorMessage(err_code)
	return out_image			

# Mean Filter
def py_p3dMeanFilter8(image_data, dimx,dimy,dimz, width =3):
	out_image = p3dFilt.malloc_uchar(dimx*dimy*dimz)
	err_code = p3dMeanFilter3D_8(image_data,out_image, dimx,dimy,dimz, width,None, None)				
	py_printErrorMessage(err_code)
	return out_image
	
# Median Filter
def py_p3dMedianFilter8(image_data, dimx,dimy,dimz, width=3):
	out_image = p3dFilt.malloc_uchar(dimx*dimy*dimz)
	err_code = p3dMedianFilter3D_8(image_data,out_image, dimx,dimy,dimz, width,None, None)				
	py_printErrorMessage(err_code)
	return out_image

# AnisotropicDiffusionFilter
def py_p3dAnisotropicDiffusionFilter8(image_data, dimx,dimy,dimz,m = 1,lambdaP = 0.01,sigma = 0.01,iterP = 20):
	out_image = p3dFilt.malloc_uchar(dimx*dimy*dimz)
	err_code = p3dAnisotropicDiffusionFilter3D_8(image_data,out_image, dimx,dimy,dimz,m,lambdaP,sigma,iterP,None, None)				
	py_printErrorMessage(err_code)
	return out_image

# BilateralFilter
def py_p3dBilateralFilter8(image_data, dimx,dimy,dimz,size,sigma_d,sigma_r,iter):
	out_image = p3dFilt.malloc_uchar(dimx*dimy*dimz)
	err_code = p3dBilateralFilter3D_8(image_data,out_image, dimx,dimy,dimz,size,sigma_d,sigma_r,iter,None, None)				
	py_printErrorMessage(err_code)
	return out_image
	
# BoinHaibelRingRemover 2D only
def py_p3dBoinHaibelRingRemover8(image_data, dimx,dimy,centerX,centerY,winsize,iterations, precision):
	out_image = p3dFilt.malloc_uchar(dimx*dimy)
	err_code = p3dBoinHaibelRingRemover2D_8(image_data,out_image, dimx,dimy,centerX,centerY,winsize,iterations, precision,None)
	py_printErrorMessage(err_code)
	return out_image

# SijbersPostnovRingRemover 2D only 
def py_p3dSijbersPostnovRingRemover8(image_data, dimx,dimy,centerX,centerY,winsize,thresh,iterations, precision):
	out_image = p3dFilt.malloc_uchar(dimx*dimy)
	err_code = p3dSijbersPostnovRingRemover2D_8(image_data,out_image, dimx,dimy,centerX,centerY,winsize,thresh,iterations, precision,None, None,None)				
	py_printErrorMessage(err_code)
	return out_image

# ClearBorderFilter
def py_p3dClearBorderFilter8(image_data, dimx,dimy,dimz,connIn = 6):
	out_image = p3dFilt.malloc_uchar(dimx*dimy)
	conn3D = 711
	if connIn == 6:
		conn3D = 711
	if connIn == 18:
		conn3D = 712
	if connIn == 26:
		conn3D = 713			
	err_code = p3dClearBorderFilter3D(image_data,out_image, dimx,dimy,dimz,conn3D,None)
	py_printErrorMessage(err_code)
	return out_image
	
# AutoThresholding
def py_p3dAutoThresholding8(image_data, dimx,dimy,dimz,methodNum):
	out_image = p3dFilt.malloc_uchar(dimx*dimy*dimz)
	thresh_val = p3dFilt.malloc_uchar(1)
	if methodNum == 1:
		err_code = p3dKittlerThresholding_8(image_data, out_image,dimx,dimy,dimz, thresh_val, None, None)
	elif methodNum == 2:
		err_code = p3dOtsuThresholding_8(image_data, out_image,dimx,dimy,dimz,thresh_val, None, None)	
	elif methodNum == 3:
		err_code = p3dPunThresholding_8(image_data, out_image,dimx,dimy,dimz,thresh_val, None, None)
	elif methodNum == 4:
		err_code = p3dRidlerThresholding_8(image_data, out_image,dimx,dimy,dimz,thresh_val, None, None)
	elif methodNum == 5:
		err_code = p3dKapurThresholding_8(image_data, out_image,dimx,dimy,dimz,thresh_val, None, None)
	elif methodNum == 6:
		err_code = p3dJohannsenThresholding_8(image_data, out_image,dimx,dimy,dimz,thresh_val, None, None)
	elif methodNum == 7:
		err_code = p3dHuangYagerThresholding_8(image_data, out_image,dimx,dimy,dimz,thresh_val, None, None)
	else:
		err_code = p3dOtsuThresholding_8(image_data, out_image,dimx,dimy,dimz, thresh_val, None, None)
	
	py_printErrorMessage(err_code)
	return out_image
		
		
#From16To8
def py_p3dFrom16To8(image_data16, dimx,dimy,dimz):
    out_image8 = p3dFilt.malloc_uchar(dimx*dimy*dimz)
    err_code = p3dFrom16To8(image_data16,out_image8,dimx,dimy,dimz,0,255,None,None)
    py_printErrorMessage(err_code)
    return out_image8
        
# CreateBinaryCircle

# CreateBinaryCylinder

# CreateBinarySphere

# GetRegionByCoords



################## Handle 16 bit

#read
def py_p3dReadRaw16(filename, dimx,dimy,dimz, little_endian = 0, is_signed = 0):
	image_data = p3dFilt.malloc_ushort(dimx*dimy*dimz)
	err_code = p3dReadRaw16(filename, image_data,dimx,dimy,dimz,little_endian, is_signed, None, None);
	py_printErrorMessage(err_code)
	return image_data

#write
def py_p3dWriteRaw16(image_data, filename, dimx,dimy,dimz, little_endian = 0, is_signed = 0):
	#image_data = p3dFilt.malloc_uchar(dimx*dimy*dimz)
	err_code = p3dWriteRaw16(image_data, filename,dimx,dimy,dimz,little_endian, is_signed, None, None);
	py_printErrorMessage(err_code)
	return image_data

def py_p3dAutoThresholding16(image_data, dimx,dimy,dimz,methodNum):
	out_image = p3dFilt.malloc_ushort(dimx*dimy*dimz)
	thresh_val = p3dFilt.malloc_ushort(1)
	if methodNum == 1:
		err_code = p3dKittlerThresholding_16(image_data, out_image,dimx,dimy,dimz, thresh_val, None, None)
	elif methodNum == 2:
		err_code = p3dOtsuThresholding_16(image_data, out_image,dimx,dimy,dimz,thresh_val, None, None)	
	elif methodNum == 3:
		err_code = p3dPunThresholding_16(image_data, out_image,dimx,dimy,dimz,thresh_val, None, None)
	elif methodNum == 4:
		err_code = p3dRidlerThresholding_16(image_data, out_image,dimx,dimy,dimz,thresh_val, None, None)
	elif methodNum == 5:
		err_code = p3dKapurThresholding_16(image_data, out_image,dimx,dimy,dimz,thresh_val, None, None)
	elif methodNum == 6:
		err_code = p3dJohannsenThresholding_16(image_data, out_image,dimx,dimy,dimz,thresh_val, None, None)
	elif methodNum == 7:
		err_code = p3dHuangYagerThresholding_16(image_data, out_image,dimx,dimy,dimz,thresh_val, None, None)
	else:
		err_code = p3dOtsuThresholding_16(image_data, out_image,dimx,dimy,dimz, thresh_val, None, None)
	
	py_printErrorMessage(err_code)
	return out_image

#void p3d_idlWriteRaw(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlGaussianFilter(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlMeanFilter(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlMedianFilter(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlAnisotropicDiffusionFilter(int, IDL_VPTR*, char*);
#IDL_VPTR p3d_idlBilateralFilter(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlBoinHaibelRingRemover(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlSijbersPostnovRingRemover(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlClearBorderFilter(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlCreateBinaryCircle(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlCreateBinaryCylinder(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlCreateBinarySphere(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlGetRegionByCoords(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlAutoThresholding(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlFrom16To8(int, IDL_VPTR*, char* );
  
  



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.1
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _p3dFilt
else:
    import _p3dFilt

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def cdata(ptr, nelements=1):
    return _p3dFilt.cdata(ptr, nelements)

def memmove(data, indata):
    return _p3dFilt.memmove(data, indata)

def malloc_uchar(*args):
    return _p3dFilt.malloc_uchar(*args)

def calloc_uchar(*args):
    return _p3dFilt.calloc_uchar(*args)

def realloc_uchar(ptr, nitems):
    return _p3dFilt.realloc_uchar(ptr, nitems)

def free_uchar(ptr):
    return _p3dFilt.free_uchar(ptr)
sizeof_uchar = _p3dFilt.sizeof_uchar

def malloc_ushort(*args):
    return _p3dFilt.malloc_ushort(*args)

def calloc_ushort(*args):
    return _p3dFilt.calloc_ushort(*args)

def realloc_ushort(ptr, nitems):
    return _p3dFilt.realloc_ushort(ptr, nitems)

def free_ushort(ptr):
    return _p3dFilt.free_ushort(ptr)
sizeof_ushort = _p3dFilt.sizeof_ushort

def p3dResetStartTime():
    return _p3dFilt.p3dResetStartTime()

def p3dGetElapsedTime():
    return _p3dFilt.p3dGetElapsedTime()

def p3dGetElapsedTime_min():
    return _p3dFilt.p3dGetElapsedTime_min()

def p3dGetElapsedTime_sec():
    return _p3dFilt.p3dGetElapsedTime_sec()
P3D_FALSE = _p3dFilt.P3D_FALSE
P3D_TRUE = _p3dFilt.P3D_TRUE
P3D_ERROR = _p3dFilt.P3D_ERROR
P3D_IO_ERROR = _p3dFilt.P3D_IO_ERROR
P3D_SUCCESS = _p3dFilt.P3D_SUCCESS
BACKGROUND = _p3dFilt.BACKGROUND
CONN6 = _p3dFilt.CONN6
CONN18 = _p3dFilt.CONN18
CONN26 = _p3dFilt.CONN26

def p3dReadRaw8(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress):
    return _p3dFilt.p3dReadRaw8(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress)

def p3dReadRaw16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log, wr_progress):
    return _p3dFilt.p3dReadRaw16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log, wr_progress)

def p3dWriteRaw8(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress):
    return _p3dFilt.p3dWriteRaw8(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress)

def p3dWriteRaw16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log, wr_progress):
    return _p3dFilt.p3dWriteRaw16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log, wr_progress)

def p3dWriteRaw32(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log, wr_progress):
    return _p3dFilt.p3dWriteRaw32(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log, wr_progress)

def p3dWriteRaw32f(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress):
    return _p3dFilt.p3dWriteRaw32f(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress)

def p3dCrop2D_8(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress):
    return _p3dFilt.p3dCrop2D_8(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress)

def p3dCrop2D_16(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress):
    return _p3dFilt.p3dCrop2D_16(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress)

def p3dCrop3D_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dCrop3D_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dCrop3D_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dCrop3D_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dZeroPadding2D_8(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress):
    return _p3dFilt.p3dZeroPadding2D_8(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress)

def p3dZeroPadding2D_16(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress):
    return _p3dFilt.p3dZeroPadding2D_16(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress)

def p3dZeroPadding3D_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dZeroPadding3D_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dZeroPadding3D_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dZeroPadding3D_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dReplicatePadding2D_8(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress):
    return _p3dFilt.p3dReplicatePadding2D_8(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress)

def p3dReplicatePadding2D_16(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress):
    return _p3dFilt.p3dReplicatePadding2D_16(arg1, arg2, arg3, arg4, arg5, wr_log, wr_progress)

def p3dReplicatePadding3D_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dReplicatePadding3D_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dReplicatePadding3D_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dReplicatePadding3D_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def _p3dZeroPadding3D_float(arg1, arg2, arg3, arg4, arg5, arg6):
    return _p3dFilt._p3dZeroPadding3D_float(arg1, arg2, arg3, arg4, arg5, arg6)

def _p3dReplicatePadding3D_float(arg1, arg2, arg3, arg4, arg5, arg6):
    return _p3dFilt._p3dReplicatePadding3D_float(arg1, arg2, arg3, arg4, arg5, arg6)

def _p3dZeroPadding3D_uchar2float(arg1, arg2, arg3, arg4, arg5, arg6):
    return _p3dFilt._p3dZeroPadding3D_uchar2float(arg1, arg2, arg3, arg4, arg5, arg6)

def _p3dReplicatePadding3D_uchar2float(arg1, arg2, arg3, arg4, arg5, arg6):
    return _p3dFilt._p3dReplicatePadding3D_uchar2float(arg1, arg2, arg3, arg4, arg5, arg6)

def _p3dZeroPadding3D_ushort2float(arg1, arg2, arg3, arg4, arg5, arg6):
    return _p3dFilt._p3dZeroPadding3D_ushort2float(arg1, arg2, arg3, arg4, arg5, arg6)

def _p3dReplicatePadding3D_ushort2float(arg1, arg2, arg3, arg4, arg5, arg6):
    return _p3dFilt._p3dReplicatePadding3D_ushort2float(arg1, arg2, arg3, arg4, arg5, arg6)

def p3dAnisotropicDiffusionFilter3D_8(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log, wr_progress):
    return _p3dFilt.p3dAnisotropicDiffusionFilter3D_8(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log, wr_progress)

def p3dAnisotropicDiffusionFilter3D_16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log, wr_progress):
    return _p3dFilt.p3dAnisotropicDiffusionFilter3D_16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log, wr_progress)

def p3dBilateralFilter3D_8(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log, wr_progress):
    return _p3dFilt.p3dBilateralFilter3D_8(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log, wr_progress)

def p3dBilateralFilter3D_16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log, wr_progress):
    return _p3dFilt.p3dBilateralFilter3D_16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log, wr_progress)

def p3dGaussianFilter3D_8(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log, wr_progress):
    return _p3dFilt.p3dGaussianFilter3D_8(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log, wr_progress)

def p3dGaussianFilter3D_16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log, wr_progress):
    return _p3dFilt.p3dGaussianFilter3D_16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log, wr_progress)

def p3dMeanFilter3D_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dMeanFilter3D_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dMeanFilter3D_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dMeanFilter3D_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dMedianFilter3D_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dMedianFilter3D_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dMedianFilter3D_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dMedianFilter3D_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dBoinHaibelRingRemover2D_8(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log):
    return _p3dFilt.p3dBoinHaibelRingRemover2D_8(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log)

def p3dBoinHaibelRingRemover2D_16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log):
    return _p3dFilt.p3dBoinHaibelRingRemover2D_16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log)

def p3dSijbersPostnovRingRemover2D_8(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, wr_log, wr_progress):
    return _p3dFilt.p3dSijbersPostnovRingRemover2D_8(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, wr_log, wr_progress)

def p3dSijbersPostnovRingRemover2D_16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, wr_log, wr_progress):
    return _p3dFilt.p3dSijbersPostnovRingRemover2D_16(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, wr_log, wr_progress)

def p3dKittlerThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dKittlerThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dKittlerThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dKittlerThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dOtsuThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dOtsuThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dOtsuThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dOtsuThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dPunThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dPunThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dPunThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dPunThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dRidlerThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dRidlerThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dRidlerThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dRidlerThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dKapurThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dKapurThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dKapurThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dKapurThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dJohannsenThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dJohannsenThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dJohannsenThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dJohannsenThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dHuangYagerThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dHuangYagerThresholding_8(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dHuangYagerThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress):
    return _p3dFilt.p3dHuangYagerThresholding_16(arg1, arg2, arg3, arg4, arg5, arg6, wr_log, wr_progress)

def p3dClearBorderFilter3D(arg1, arg2, arg3, arg4, arg5, arg6, wr_log):
    return _p3dFilt.p3dClearBorderFilter3D(arg1, arg2, arg3, arg4, arg5, arg6, wr_log)

def p3dGetRegionByCoords3D(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log):
    return _p3dFilt.p3dGetRegionByCoords3D(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, wr_log)

def p3dCreateBinaryCircle(arg1, arg2, arg3, arg4, arg5, arg6, wr_log):
    return _p3dFilt.p3dCreateBinaryCircle(arg1, arg2, arg3, arg4, arg5, arg6, wr_log)

def p3dCreateBinaryCylinder(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log):
    return _p3dFilt.p3dCreateBinaryCylinder(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log)

def p3dCreateBinarySphere(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, wr_log):
    return _p3dFilt.p3dCreateBinarySphere(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, wr_log)

def p3dFrom16To8(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log, wr_progress):
    return _p3dFilt.p3dFrom16To8(arg1, arg2, arg3, arg4, arg5, arg6, arg7, wr_log, wr_progress)
class coords_qelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    item = property(_p3dFilt.coords_qelem_t_item_get, _p3dFilt.coords_qelem_t_item_set)
    next = property(_p3dFilt.coords_qelem_t_next_get, _p3dFilt.coords_qelem_t_next_set)

    def __init__(self):
        _p3dFilt.coords_qelem_t_swiginit(self, _p3dFilt.new_coords_qelem_t())
    __swig_destroy__ = _p3dFilt.delete_coords_qelem_t

# Register coords_qelem_t in _p3dFilt:
_p3dFilt.coords_qelem_t_swigregister(coords_qelem_t)

class coords_q_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    tail = property(_p3dFilt.coords_q_t_tail_get, _p3dFilt.coords_q_t_tail_set)
    head = property(_p3dFilt.coords_q_t_head_get, _p3dFilt.coords_q_t_head_set)

    def __init__(self):
        _p3dFilt.coords_q_t_swiginit(self, _p3dFilt.new_coords_q_t())
    __swig_destroy__ = _p3dFilt.delete_coords_q_t

# Register coords_q_t in _p3dFilt:
_p3dFilt.coords_q_t_swigregister(coords_q_t)


def coords_queue_init(queue):
    return _p3dFilt.coords_queue_init(queue)

def coords_queue_push(queue, elem):
    return _p3dFilt.coords_queue_push(queue, elem)

def coords_queue_pop(queue):
    return _p3dFilt.coords_queue_pop(queue)

def coords_queue_isempty(queue):
    return _p3dFilt.coords_queue_isempty(queue)
class coords_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_p3dFilt.coords_t_x_get, _p3dFilt.coords_t_x_set)
    y = property(_p3dFilt.coords_t_y_get, _p3dFilt.coords_t_y_set)
    z = property(_p3dFilt.coords_t_z_get, _p3dFilt.coords_t_z_set)

    def __init__(self):
        _p3dFilt.coords_t_swiginit(self, _p3dFilt.new_coords_t())
    __swig_destroy__ = _p3dFilt.delete_coords_t

# Register coords_t in _p3dFilt:
_p3dFilt.coords_t_swigregister(coords_t)

class fcoords_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_p3dFilt.fcoords_t_x_get, _p3dFilt.fcoords_t_x_set)
    y = property(_p3dFilt.fcoords_t_y_get, _p3dFilt.fcoords_t_y_set)
    z = property(_p3dFilt.fcoords_t_z_get, _p3dFilt.fcoords_t_z_set)

    def __init__(self):
        _p3dFilt.fcoords_t_swiginit(self, _p3dFilt.new_fcoords_t())
    __swig_destroy__ = _p3dFilt.delete_fcoords_t

# Register fcoords_t in _p3dFilt:
_p3dFilt.fcoords_t_swigregister(fcoords_t)


def _isPath(arg1):
    return _p3dFilt._isPath(arg1)

def p3dCartesian2polar_8(in_im, out_im, dimx, dimy, centerX, centerY, precision, polarX):
    return _p3dFilt.p3dCartesian2polar_8(in_im, out_im, dimx, dimy, centerX, centerY, precision, polarX)

def p3dCartesian2polar_16(in_im, out_im, dimx, dimy, centerX, centerY, precision, polarX):
    return _p3dFilt.p3dCartesian2polar_16(in_im, out_im, dimx, dimy, centerX, centerY, precision, polarX)

def p3dPolar2cartesian_8(in_im, out_im, polarX, centerX, centerY, original_dimx, original_dimy):
    return _p3dFilt.p3dPolar2cartesian_8(in_im, out_im, polarX, centerX, centerY, original_dimx, original_dimy)

def p3dPolar2cartesian_16(in_im, out_im, polarX, centerX, centerY, original_dimx, original_dimy):
    return _p3dFilt.p3dPolar2cartesian_16(in_im, out_im, polarX, centerX, centerY, original_dimx, original_dimy)





# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.1
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _p3dSkel
else:
    import _p3dSkel

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def cdata(ptr, nelements=1):
    return _p3dSkel.cdata(ptr, nelements)

def memmove(data, indata):
    return _p3dSkel.memmove(data, indata)

def malloc_uchar(*args):
    return _p3dSkel.malloc_uchar(*args)

def calloc_uchar(*args):
    return _p3dSkel.calloc_uchar(*args)

def realloc_uchar(ptr, nitems):
    return _p3dSkel.realloc_uchar(ptr, nitems)

def free_uchar(ptr):
    return _p3dSkel.free_uchar(ptr)
sizeof_uchar = _p3dSkel.sizeof_uchar

def malloc_ushort(*args):
    return _p3dSkel.malloc_ushort(*args)

def calloc_ushort(*args):
    return _p3dSkel.calloc_ushort(*args)

def realloc_ushort(ptr, nitems):
    return _p3dSkel.realloc_ushort(ptr, nitems)

def free_ushort(ptr):
    return _p3dSkel.free_ushort(ptr)
sizeof_ushort = _p3dSkel.sizeof_ushort

def malloc_PSkeletonStats(*args):
    return _p3dSkel.malloc_PSkeletonStats(*args)

def calloc_PSkeletonStats(*args):
    return _p3dSkel.calloc_PSkeletonStats(*args)

def realloc_PSkeletonStats(ptr, nitems):
    return _p3dSkel.realloc_PSkeletonStats(ptr, nitems)

def free_PSkeletonStats(ptr):
    return _p3dSkel.free_PSkeletonStats(ptr)
sizeof_PSkeletonStats = _p3dSkel.sizeof_PSkeletonStats
P3D_FALSE = _p3dSkel.P3D_FALSE
P3D_TRUE = _p3dSkel.P3D_TRUE
P3D_ERROR = _p3dSkel.P3D_ERROR
P3D_SUCCESS = _p3dSkel.P3D_SUCCESS
BACKGROUND = _p3dSkel.BACKGROUND
NODE_LABEL = _p3dSkel.NODE_LABEL
END_LABEL = _p3dSkel.END_LABEL
ISOLATED_LABEL = _p3dSkel.ISOLATED_LABEL
NODETONODE_LABEL = _p3dSkel.NODETONODE_LABEL
NODETOEND_LABEL = _p3dSkel.NODETOEND_LABEL
ENDTOEND_LABEL = _p3dSkel.ENDTOEND_LABEL
class SkeletonStats(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ConnectivityDensity = property(_p3dSkel.SkeletonStats_ConnectivityDensity_get, _p3dSkel.SkeletonStats_ConnectivityDensity_set)
    CoordinationNumber = property(_p3dSkel.SkeletonStats_CoordinationNumber_get, _p3dSkel.SkeletonStats_CoordinationNumber_set)
    End_Counter = property(_p3dSkel.SkeletonStats_End_Counter_get, _p3dSkel.SkeletonStats_End_Counter_set)
    End_Width = property(_p3dSkel.SkeletonStats_End_Width_get, _p3dSkel.SkeletonStats_End_Width_set)
    EndToEnd_Counter = property(_p3dSkel.SkeletonStats_EndToEnd_Counter_get, _p3dSkel.SkeletonStats_EndToEnd_Counter_set)
    EndToEnd_Length = property(_p3dSkel.SkeletonStats_EndToEnd_Length_get, _p3dSkel.SkeletonStats_EndToEnd_Length_set)
    EndToEnd_MaxWidth = property(_p3dSkel.SkeletonStats_EndToEnd_MaxWidth_get, _p3dSkel.SkeletonStats_EndToEnd_MaxWidth_set)
    EndToEnd_MeanWidth = property(_p3dSkel.SkeletonStats_EndToEnd_MeanWidth_get, _p3dSkel.SkeletonStats_EndToEnd_MeanWidth_set)
    EndToEnd_MinWidth = property(_p3dSkel.SkeletonStats_EndToEnd_MinWidth_get, _p3dSkel.SkeletonStats_EndToEnd_MinWidth_set)
    NodeToEnd_Counter = property(_p3dSkel.SkeletonStats_NodeToEnd_Counter_get, _p3dSkel.SkeletonStats_NodeToEnd_Counter_set)
    NodeToEnd_Length = property(_p3dSkel.SkeletonStats_NodeToEnd_Length_get, _p3dSkel.SkeletonStats_NodeToEnd_Length_set)
    NodeToEnd_MaxWidth = property(_p3dSkel.SkeletonStats_NodeToEnd_MaxWidth_get, _p3dSkel.SkeletonStats_NodeToEnd_MaxWidth_set)
    NodeToEnd_MeanWidth = property(_p3dSkel.SkeletonStats_NodeToEnd_MeanWidth_get, _p3dSkel.SkeletonStats_NodeToEnd_MeanWidth_set)
    NodeToEnd_MinWidth = property(_p3dSkel.SkeletonStats_NodeToEnd_MinWidth_get, _p3dSkel.SkeletonStats_NodeToEnd_MinWidth_set)
    NodeToNode_Counter = property(_p3dSkel.SkeletonStats_NodeToNode_Counter_get, _p3dSkel.SkeletonStats_NodeToNode_Counter_set)
    NodeToNode_Length = property(_p3dSkel.SkeletonStats_NodeToNode_Length_get, _p3dSkel.SkeletonStats_NodeToNode_Length_set)
    NodeToNode_MaxWidth = property(_p3dSkel.SkeletonStats_NodeToNode_MaxWidth_get, _p3dSkel.SkeletonStats_NodeToNode_MaxWidth_set)
    NodeToNode_MeanWidth = property(_p3dSkel.SkeletonStats_NodeToNode_MeanWidth_get, _p3dSkel.SkeletonStats_NodeToNode_MeanWidth_set)
    NodeToNode_MinWidth = property(_p3dSkel.SkeletonStats_NodeToNode_MinWidth_get, _p3dSkel.SkeletonStats_NodeToNode_MinWidth_set)
    Node_Counter = property(_p3dSkel.SkeletonStats_Node_Counter_get, _p3dSkel.SkeletonStats_Node_Counter_set)
    Node_Width = property(_p3dSkel.SkeletonStats_Node_Width_get, _p3dSkel.SkeletonStats_Node_Width_set)

    def __init__(self):
        _p3dSkel.SkeletonStats_swiginit(self, _p3dSkel.new_SkeletonStats())
    __swig_destroy__ = _p3dSkel.delete_SkeletonStats

# Register SkeletonStats in _p3dSkel:
_p3dSkel.SkeletonStats_swigregister(SkeletonStats)


def p3dGVFSkeletonization(in_im, skl_im, dimx, dimy, dimz, mu, eps, thresh, scale, wr_log):
    return _p3dSkel.p3dGVFSkeletonization(in_im, skl_im, dimx, dimy, dimz, mu, eps, thresh, scale, wr_log)

def p3dThinningSkeletonization(in_im, out_im, dimx, dimy, dimz, wr_log):
    return _p3dSkel.p3dThinningSkeletonization(in_im, out_im, dimx, dimy, dimz, wr_log)

def p3dLKCSkeletonization(in_im, out_im, dimx, dimy, dimz, wr_log):
    return _p3dSkel.p3dLKCSkeletonization(in_im, out_im, dimx, dimy, dimz, wr_log)

def p3dSimpleSkeletonPruning(in_im, out_im, dimx, dimy, dimz, thresh, wr_log):
    return _p3dSkel.p3dSimpleSkeletonPruning(in_im, out_im, dimx, dimy, dimz, thresh, wr_log)

def p3dIterativeSkeletonPruning(in_im, out_im, dimx, dimy, dimz, thresh, wr_log):
    return _p3dSkel.p3dIterativeSkeletonPruning(in_im, out_im, dimx, dimy, dimz, thresh, wr_log)

def p3dUltimateSkeletonPruning(in_im, out_im, dimx, dimy, dimz, iterative, wr_log):
    return _p3dSkel.p3dUltimateSkeletonPruning(in_im, out_im, dimx, dimy, dimz, iterative, wr_log)

def p3dSkeletonLabeling(in_im, out_im, dimx, dimy, dimz, wr_log):
    return _p3dSkel.p3dSkeletonLabeling(in_im, out_im, dimx, dimy, dimz, wr_log)

def p3dSkeletonAnalysis(vol_im, skl_im, out_stats, nodes_im, pores_im, ends_im, throats_im, dimx, dimy, dimz, merging_factor, tortuosity_depth, voxelsize, wr_log):
    return _p3dSkel.p3dSkeletonAnalysis(vol_im, skl_im, out_stats, nodes_im, pores_im, ends_im, throats_im, dimx, dimy, dimz, merging_factor, tortuosity_depth, voxelsize, wr_log)

def p3dSkeletonAnalysis_2(vol_im, skl_im, out_stats, nodes_im, pores_im, ends_im, throats_im, dimx, dimy, dimz, merging_factor, tortuosity_depth, voxelsize, wr_log):
    return _p3dSkel.p3dSkeletonAnalysis_2(vol_im, skl_im, out_stats, nodes_im, pores_im, ends_im, throats_im, dimx, dimy, dimz, merging_factor, tortuosity_depth, voxelsize, wr_log)

def p3dSkeletonAnalysisFeasibility(in_im, sk_im, ratio, dimx, dimy, dimz, wr_log):
    return _p3dSkel.p3dSkeletonAnalysisFeasibility(in_im, sk_im, ratio, dimx, dimy, dimz, wr_log)

def p3dResetStartTime():
    return _p3dSkel.p3dResetStartTime()

def p3dGetElapsedTime():
    return _p3dSkel.p3dGetElapsedTime()

def p3dGetElapsedTime_min():
    return _p3dSkel.p3dGetElapsedTime_min()

def p3dGetElapsedTime_sec():
    return _p3dSkel.p3dGetElapsedTime_sec()
class bb_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    min_x = property(_p3dSkel.bb_t_min_x_get, _p3dSkel.bb_t_min_x_set)
    max_x = property(_p3dSkel.bb_t_max_x_get, _p3dSkel.bb_t_max_x_set)
    min_y = property(_p3dSkel.bb_t_min_y_get, _p3dSkel.bb_t_min_y_set)
    max_y = property(_p3dSkel.bb_t_max_y_get, _p3dSkel.bb_t_max_y_set)
    min_z = property(_p3dSkel.bb_t_min_z_get, _p3dSkel.bb_t_min_z_set)
    max_z = property(_p3dSkel.bb_t_max_z_get, _p3dSkel.bb_t_max_z_set)

    def __init__(self):
        _p3dSkel.bb_t_swiginit(self, _p3dSkel.new_bb_t())
    __swig_destroy__ = _p3dSkel.delete_bb_t

# Register bb_t in _p3dSkel:
_p3dSkel.bb_t_swigregister(bb_t)


def p3dConnectedComponentsLabeling(in_rev, out_rev, numOfConnectedComponents, volumes, boundingBoxes, dimx, dimy, dimz, conn, skip_borders):
    return _p3dSkel.p3dConnectedComponentsLabeling(in_rev, out_rev, numOfConnectedComponents, volumes, boundingBoxes, dimx, dimy, dimz, conn, skip_borders)

def p3dGetMaxVolumeRegion(in_rev, out_rev, dimx, dimy, dimz, conn):
    return _p3dSkel.p3dGetMaxVolumeRegion(in_rev, out_rev, dimx, dimy, dimz, conn)
class coords_lelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    elem = property(_p3dSkel.coords_lelem_t_elem_get, _p3dSkel.coords_lelem_t_elem_set)
    next = property(_p3dSkel.coords_lelem_t_next_get, _p3dSkel.coords_lelem_t_next_set)

    def __init__(self):
        _p3dSkel.coords_lelem_t_swiginit(self, _p3dSkel.new_coords_lelem_t())
    __swig_destroy__ = _p3dSkel.delete_coords_lelem_t

# Register coords_lelem_t in _p3dSkel:
_p3dSkel.coords_lelem_t_swigregister(coords_lelem_t)


def coords_list_init(arg1):
    return _p3dSkel.coords_list_init(arg1)

def coords_list_push(arg1, arg2):
    return _p3dSkel.coords_list_push(arg1, arg2)

def coords_list_pop(arg1):
    return _p3dSkel.coords_list_pop(arg1)

def coords_list_isempty(arg1):
    return _p3dSkel.coords_list_isempty(arg1)

def coords_list_toarray(arg1, arg2):
    return _p3dSkel.coords_list_toarray(arg1, arg2)
class coords_qelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    item = property(_p3dSkel.coords_qelem_t_item_get, _p3dSkel.coords_qelem_t_item_set)
    next = property(_p3dSkel.coords_qelem_t_next_get, _p3dSkel.coords_qelem_t_next_set)

    def __init__(self):
        _p3dSkel.coords_qelem_t_swiginit(self, _p3dSkel.new_coords_qelem_t())
    __swig_destroy__ = _p3dSkel.delete_coords_qelem_t

# Register coords_qelem_t in _p3dSkel:
_p3dSkel.coords_qelem_t_swigregister(coords_qelem_t)

class coords_queue_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    tail = property(_p3dSkel.coords_queue_t_tail_get, _p3dSkel.coords_queue_t_tail_set)
    head = property(_p3dSkel.coords_queue_t_head_get, _p3dSkel.coords_queue_t_head_set)

    def __init__(self):
        _p3dSkel.coords_queue_t_swiginit(self, _p3dSkel.new_coords_queue_t())
    __swig_destroy__ = _p3dSkel.delete_coords_queue_t

# Register coords_queue_t in _p3dSkel:
_p3dSkel.coords_queue_t_swigregister(coords_queue_t)


def coords_queue_init(queue):
    return _p3dSkel.coords_queue_init(queue)

def coords_queue_push(queue, elem):
    return _p3dSkel.coords_queue_push(queue, elem)

def coords_queue_pop(queue):
    return _p3dSkel.coords_queue_pop(queue)

def coords_queue_isempty(queue):
    return _p3dSkel.coords_queue_isempty(queue)
class coords_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_p3dSkel.coords_t_x_get, _p3dSkel.coords_t_x_set)
    y = property(_p3dSkel.coords_t_y_get, _p3dSkel.coords_t_y_set)
    z = property(_p3dSkel.coords_t_z_get, _p3dSkel.coords_t_z_set)

    def __init__(self):
        _p3dSkel.coords_t_swiginit(self, _p3dSkel.new_coords_t())
    __swig_destroy__ = _p3dSkel.delete_coords_t

# Register coords_t in _p3dSkel:
_p3dSkel.coords_t_swigregister(coords_t)

class fcoords_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_p3dSkel.fcoords_t_x_get, _p3dSkel.fcoords_t_x_set)
    y = property(_p3dSkel.fcoords_t_y_get, _p3dSkel.fcoords_t_y_set)
    z = property(_p3dSkel.fcoords_t_z_get, _p3dSkel.fcoords_t_z_set)

    def __init__(self):
        _p3dSkel.fcoords_t_swiginit(self, _p3dSkel.new_fcoords_t())
    __swig_destroy__ = _p3dSkel.delete_fcoords_t

# Register fcoords_t in _p3dSkel:
_p3dSkel.fcoords_t_swigregister(fcoords_t)

class fcoords_lelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    elem = property(_p3dSkel.fcoords_lelem_t_elem_get, _p3dSkel.fcoords_lelem_t_elem_set)
    next = property(_p3dSkel.fcoords_lelem_t_next_get, _p3dSkel.fcoords_lelem_t_next_set)

    def __init__(self):
        _p3dSkel.fcoords_lelem_t_swiginit(self, _p3dSkel.new_fcoords_lelem_t())
    __swig_destroy__ = _p3dSkel.delete_fcoords_lelem_t

# Register fcoords_lelem_t in _p3dSkel:
_p3dSkel.fcoords_lelem_t_swigregister(fcoords_lelem_t)


def fcoords_list_init(arg1):
    return _p3dSkel.fcoords_list_init(arg1)

def fcoords_list_push(arg1, arg2):
    return _p3dSkel.fcoords_list_push(arg1, arg2)

def fcoords_list_pop(arg1):
    return _p3dSkel.fcoords_list_pop(arg1)

def fcoords_list_isempty(arg1):
    return _p3dSkel.fcoords_list_isempty(arg1)

def fcoords_list_toarray(arg1, arg2):
    return _p3dSkel.fcoords_list_toarray(arg1, arg2)

def p3dSquaredEuclideanDT(in_im, out_im, dimx, dimy, dimz):
    return _p3dSkel.p3dSquaredEuclideanDT(in_im, out_im, dimx, dimy, dimz)

def p3dThinning(im, dimx, dimy, dimz):
    return _p3dSkel.p3dThinning(im, dimx, dimy, dimz)
class uint_lelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ct = property(_p3dSkel.uint_lelem_t_ct_get, _p3dSkel.uint_lelem_t_ct_set)
    next = property(_p3dSkel.uint_lelem_t_next_get, _p3dSkel.uint_lelem_t_next_set)

    def __init__(self):
        _p3dSkel.uint_lelem_t_swiginit(self, _p3dSkel.new_uint_lelem_t())
    __swig_destroy__ = _p3dSkel.delete_uint_lelem_t

# Register uint_lelem_t in _p3dSkel:
_p3dSkel.uint_lelem_t_swigregister(uint_lelem_t)


def uint_list_init(list):
    return _p3dSkel.uint_list_init(list)

def uint_list_add(list, ct):
    return _p3dSkel.uint_list_add(list, ct)

def uint_list_isempty(list):
    return _p3dSkel.uint_list_isempty(list)

def uint_list_toarray(list, numel):
    return _p3dSkel.uint_list_toarray(list, numel)

def uint_list_clear(list):
    return _p3dSkel.uint_list_clear(list)

def p3dComputeCoreSkeleton(crit_point_list, skel_point_list, gvf_x, gvf_y, gvf_z, dimx, dimy, dimz, step, close):
    return _p3dSkel.p3dComputeCoreSkeleton(crit_point_list, skel_point_list, gvf_x, gvf_y, gvf_z, dimx, dimy, dimz, step, close)

def p3dComputeEigenVal(H, S, eigen_re, eigen_im, n):
    return _p3dSkel.p3dComputeEigenVal(H, S, eigen_re, eigen_im, n)

def p3dComputeHierarchicalSkeleton(highDiv_point_list, skel_point_list, gvf_x, gvf_y, gvf_z, dimx, dimy, dimz, step, close_dist):
    return _p3dSkel.p3dComputeHierarchicalSkeleton(highDiv_point_list, skel_point_list, gvf_x, gvf_y, gvf_z, dimx, dimy, dimz, step, close_dist)

def p3dGetCriticalPoints(in_im, gvf_x, gvf_y, gvf_z, dimx, dimy, dimz, crit_point_list):
    return _p3dSkel.p3dGetCriticalPoints(in_im, gvf_x, gvf_y, gvf_z, dimx, dimy, dimz, crit_point_list)

def p3dClassifyCriticalPoints(crit_point_list, gvf_x, gvf_y, gvf_z, dimx, dimy, dimz):
    return _p3dSkel.p3dClassifyCriticalPoints(crit_point_list, gvf_x, gvf_y, gvf_z, dimx, dimy, dimz)
class crit_point_lelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    elem = property(_p3dSkel.crit_point_lelem_t_elem_get, _p3dSkel.crit_point_lelem_t_elem_set)
    next = property(_p3dSkel.crit_point_lelem_t_next_get, _p3dSkel.crit_point_lelem_t_next_set)

    def __init__(self):
        _p3dSkel.crit_point_lelem_t_swiginit(self, _p3dSkel.new_crit_point_lelem_t())
    __swig_destroy__ = _p3dSkel.delete_crit_point_lelem_t

# Register crit_point_lelem_t in _p3dSkel:
_p3dSkel.crit_point_lelem_t_swigregister(crit_point_lelem_t)


def crit_point_list_init(list):
    return _p3dSkel.crit_point_list_init(list)

def crit_point_list_push(list, item):
    return _p3dSkel.crit_point_list_push(list, item)

def crit_point_list_pop(list):
    return _p3dSkel.crit_point_list_pop(list)

def crit_point_list_isempty(list):
    return _p3dSkel.crit_point_list_isempty(list)
class crit_point_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_p3dSkel.crit_point_t_x_get, _p3dSkel.crit_point_t_x_set)
    y = property(_p3dSkel.crit_point_t_y_get, _p3dSkel.crit_point_t_y_set)
    z = property(_p3dSkel.crit_point_t_z_get, _p3dSkel.crit_point_t_z_set)
    type = property(_p3dSkel.crit_point_t_type_get, _p3dSkel.crit_point_t_type_set)
    eval0 = property(_p3dSkel.crit_point_t_eval0_get, _p3dSkel.crit_point_t_eval0_set)
    eval1 = property(_p3dSkel.crit_point_t_eval1_get, _p3dSkel.crit_point_t_eval1_set)
    eval2 = property(_p3dSkel.crit_point_t_eval2_get, _p3dSkel.crit_point_t_eval2_set)
    evect0_x = property(_p3dSkel.crit_point_t_evect0_x_get, _p3dSkel.crit_point_t_evect0_x_set)
    evect0_y = property(_p3dSkel.crit_point_t_evect0_y_get, _p3dSkel.crit_point_t_evect0_y_set)
    evect0_z = property(_p3dSkel.crit_point_t_evect0_z_get, _p3dSkel.crit_point_t_evect0_z_set)
    evect1_x = property(_p3dSkel.crit_point_t_evect1_x_get, _p3dSkel.crit_point_t_evect1_x_set)
    evect1_y = property(_p3dSkel.crit_point_t_evect1_y_get, _p3dSkel.crit_point_t_evect1_y_set)
    evect1_z = property(_p3dSkel.crit_point_t_evect1_z_get, _p3dSkel.crit_point_t_evect1_z_set)
    evect2_x = property(_p3dSkel.crit_point_t_evect2_x_get, _p3dSkel.crit_point_t_evect2_x_set)
    evect2_y = property(_p3dSkel.crit_point_t_evect2_y_get, _p3dSkel.crit_point_t_evect2_y_set)
    evect2_z = property(_p3dSkel.crit_point_t_evect2_z_get, _p3dSkel.crit_point_t_evect2_z_set)

    def __init__(self):
        _p3dSkel.crit_point_t_swiginit(self, _p3dSkel.new_crit_point_t())
    __swig_destroy__ = _p3dSkel.delete_crit_point_t

# Register crit_point_t in _p3dSkel:
_p3dSkel.crit_point_t_swigregister(crit_point_t)


def p3dGetHighDivPoints(in_im, gvf_x, gvf_y, gvf_z, dimx, dimy, dimz, thresh, highDiv_point_list):
    return _p3dSkel.p3dGetHighDivPoints(in_im, gvf_x, gvf_y, gvf_z, dimx, dimy, dimz, thresh, highDiv_point_list)

def p3dGVF(in_im, out_x, out_y, out_z, dimx, dimy, dimz, mu, eps):
    return _p3dSkel.p3dGVF(in_im, out_x, out_y, out_z, dimx, dimy, dimz, mu, eps)
class highDiv_point_lelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    elem = property(_p3dSkel.highDiv_point_lelem_t_elem_get, _p3dSkel.highDiv_point_lelem_t_elem_set)
    next = property(_p3dSkel.highDiv_point_lelem_t_next_get, _p3dSkel.highDiv_point_lelem_t_next_set)

    def __init__(self):
        _p3dSkel.highDiv_point_lelem_t_swiginit(self, _p3dSkel.new_highDiv_point_lelem_t())
    __swig_destroy__ = _p3dSkel.delete_highDiv_point_lelem_t

# Register highDiv_point_lelem_t in _p3dSkel:
_p3dSkel.highDiv_point_lelem_t_swigregister(highDiv_point_lelem_t)


def highDiv_point_list_init(list):
    return _p3dSkel.highDiv_point_list_init(list)

def highDiv_point_list_push(list, item):
    return _p3dSkel.highDiv_point_list_push(list, item)

def highDiv_point_list_pop(list):
    return _p3dSkel.highDiv_point_list_pop(list)

def highDiv_point_list_isempty(list):
    return _p3dSkel.highDiv_point_list_isempty(list)

def highDiv_point_list_toarray(list, numel):
    return _p3dSkel.highDiv_point_list_toarray(list, numel)
class highDiv_point_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_p3dSkel.highDiv_point_t_x_get, _p3dSkel.highDiv_point_t_x_set)
    y = property(_p3dSkel.highDiv_point_t_y_get, _p3dSkel.highDiv_point_t_y_set)
    z = property(_p3dSkel.highDiv_point_t_z_get, _p3dSkel.highDiv_point_t_z_set)
    div = property(_p3dSkel.highDiv_point_t_div_get, _p3dSkel.highDiv_point_t_div_set)

    def __init__(self):
        _p3dSkel.highDiv_point_t_swiginit(self, _p3dSkel.new_highDiv_point_t())
    __swig_destroy__ = _p3dSkel.delete_highDiv_point_t

# Register highDiv_point_t in _p3dSkel:
_p3dSkel.highDiv_point_t_swigregister(highDiv_point_t)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.1
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _p3dBlob
else:
    import _p3dBlob

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def cdata(ptr, nelements=1):
    return _p3dBlob.cdata(ptr, nelements)

def memmove(data, indata):
    return _p3dBlob.memmove(data, indata)

def malloc_uchar(*args):
    return _p3dBlob.malloc_uchar(*args)

def calloc_uchar(*args):
    return _p3dBlob.calloc_uchar(*args)

def realloc_uchar(ptr, nitems):
    return _p3dBlob.realloc_uchar(ptr, nitems)

def free_uchar(ptr):
    return _p3dBlob.free_uchar(ptr)
sizeof_uchar = _p3dBlob.sizeof_uchar

def malloc_ushort(*args):
    return _p3dBlob.malloc_ushort(*args)

def calloc_ushort(*args):
    return _p3dBlob.calloc_ushort(*args)

def realloc_ushort(ptr, nitems):
    return _p3dBlob.realloc_ushort(ptr, nitems)

def free_ushort(ptr):
    return _p3dBlob.free_ushort(ptr)
sizeof_ushort = _p3dBlob.sizeof_ushort

def malloc_BasicStat(*args):
    return _p3dBlob.malloc_BasicStat(*args)

def calloc_BasicStat(*args):
    return _p3dBlob.calloc_BasicStat(*args)

def realloc_BasicStat(ptr, nitems):
    return _p3dBlob.realloc_BasicStat(ptr, nitems)

def free_BasicStat(ptr):
    return _p3dBlob.free_BasicStat(ptr)
sizeof_BasicStat = _p3dBlob.sizeof_BasicStat

def malloc_AnisotropyStat(*args):
    return _p3dBlob.malloc_AnisotropyStat(*args)

def calloc_AnisotropyStat(*args):
    return _p3dBlob.calloc_AnisotropyStat(*args)

def realloc_AnisotropyStat(ptr, nitems):
    return _p3dBlob.realloc_AnisotropyStat(ptr, nitems)

def free_AnisotropyStat(ptr):
    return _p3dBlob.free_AnisotropyStat(ptr)
sizeof_AnisotropyStat = _p3dBlob.sizeof_AnisotropyStat

def malloc_MorphometricStat(*args):
    return _p3dBlob.malloc_MorphometricStat(*args)

def calloc_MorphometricStat(*args):
    return _p3dBlob.calloc_MorphometricStat(*args)

def realloc_MorphometricStat(ptr, nitems):
    return _p3dBlob.realloc_MorphometricStat(ptr, nitems)

def free_MorphometricStat(ptr):
    return _p3dBlob.free_MorphometricStat(ptr)
sizeof_MorphometricStat = _p3dBlob.sizeof_MorphometricStat

def malloc_TextureStat(*args):
    return _p3dBlob.malloc_TextureStat(*args)

def calloc_TextureStat(*args):
    return _p3dBlob.calloc_TextureStat(*args)

def realloc_TextureStat(ptr, nitems):
    return _p3dBlob.realloc_TextureStat(ptr, nitems)

def free_TextureStat(ptr):
    return _p3dBlob.free_TextureStat(ptr)
sizeof_TextureStat = _p3dBlob.sizeof_TextureStat

def malloc_BlobStat(*args):
    return _p3dBlob.malloc_BlobStat(*args)

def calloc_BlobStat(*args):
    return _p3dBlob.calloc_BlobStat(*args)

def realloc_BlobStat(ptr, nitems):
    return _p3dBlob.realloc_BlobStat(ptr, nitems)

def free_BlobStat(ptr):
    return _p3dBlob.free_BlobStat(ptr)
sizeof_BlobStat = _p3dBlob.sizeof_BlobStat

def PrintBlobStruct(out_stats, filename):
    return _p3dBlob.PrintBlobStruct(out_stats, filename)

def invert_vol(in_im, dimx, dimy, dimz):
    return _p3dBlob.invert_vol(in_im, dimx, dimy, dimz)

def invert_vol_16(in_im, dimx, dimy, dimz):
    return _p3dBlob.invert_vol_16(in_im, dimx, dimy, dimz)
P3D_FALSE = _p3dBlob.P3D_FALSE
P3D_TRUE = _p3dBlob.P3D_TRUE
P3D_ERROR = _p3dBlob.P3D_ERROR
P3D_SUCCESS = _p3dBlob.P3D_SUCCESS
BACKGROUND = _p3dBlob.BACKGROUND
CONN4 = _p3dBlob.CONN4
CONN8 = _p3dBlob.CONN8
CONN6 = _p3dBlob.CONN6
CONN18 = _p3dBlob.CONN18
CONN26 = _p3dBlob.CONN26
class BlobStats(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    aspect_ratio = property(_p3dBlob.BlobStats_aspect_ratio_get, _p3dBlob.BlobStats_aspect_ratio_set)
    blobCount = property(_p3dBlob.BlobStats_blobCount_get, _p3dBlob.BlobStats_blobCount_set)
    max_sph = property(_p3dBlob.BlobStats_max_sph_get, _p3dBlob.BlobStats_max_sph_set)
    eq_sph = property(_p3dBlob.BlobStats_eq_sph_get, _p3dBlob.BlobStats_eq_sph_set)
    l_min = property(_p3dBlob.BlobStats_l_min_get, _p3dBlob.BlobStats_l_min_set)
    l_max = property(_p3dBlob.BlobStats_l_max_get, _p3dBlob.BlobStats_l_max_set)
    sphericity = property(_p3dBlob.BlobStats_sphericity_get, _p3dBlob.BlobStats_sphericity_set)
    extent = property(_p3dBlob.BlobStats_extent_get, _p3dBlob.BlobStats_extent_set)
    volume = property(_p3dBlob.BlobStats_volume_get, _p3dBlob.BlobStats_volume_set)

    def __init__(self):
        _p3dBlob.BlobStats_swiginit(self, _p3dBlob.new_BlobStats())
    __swig_destroy__ = _p3dBlob.delete_BlobStats

# Register BlobStats in _p3dBlob:
_p3dBlob.BlobStats_swigregister(BlobStats)

class MorphometricStats(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BvTv = property(_p3dBlob.MorphometricStats_BvTv_get, _p3dBlob.MorphometricStats_BvTv_set)
    BsBv = property(_p3dBlob.MorphometricStats_BsBv_get, _p3dBlob.MorphometricStats_BsBv_set)
    TbN = property(_p3dBlob.MorphometricStats_TbN_get, _p3dBlob.MorphometricStats_TbN_set)
    TbTh = property(_p3dBlob.MorphometricStats_TbTh_get, _p3dBlob.MorphometricStats_TbTh_set)
    TbSp = property(_p3dBlob.MorphometricStats_TbSp_get, _p3dBlob.MorphometricStats_TbSp_set)

    def __init__(self):
        _p3dBlob.MorphometricStats_swiginit(self, _p3dBlob.new_MorphometricStats())
    __swig_destroy__ = _p3dBlob.delete_MorphometricStats

# Register MorphometricStats in _p3dBlob:
_p3dBlob.MorphometricStats_swigregister(MorphometricStats)

class AnisotropyStats(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    I = property(_p3dBlob.AnisotropyStats_I_get, _p3dBlob.AnisotropyStats_I_set)
    E = property(_p3dBlob.AnisotropyStats_E_get, _p3dBlob.AnisotropyStats_E_set)

    def __init__(self):
        _p3dBlob.AnisotropyStats_swiginit(self, _p3dBlob.new_AnisotropyStats())
    __swig_destroy__ = _p3dBlob.delete_AnisotropyStats

# Register AnisotropyStats in _p3dBlob:
_p3dBlob.AnisotropyStats_swigregister(AnisotropyStats)

class BasicStats(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Vv = property(_p3dBlob.BasicStats_Vv_get, _p3dBlob.BasicStats_Vv_set)
    Cv = property(_p3dBlob.BasicStats_Cv_get, _p3dBlob.BasicStats_Cv_set)
    Mv = property(_p3dBlob.BasicStats_Mv_get, _p3dBlob.BasicStats_Mv_set)
    Sv = property(_p3dBlob.BasicStats_Sv_get, _p3dBlob.BasicStats_Sv_set)

    def __init__(self):
        _p3dBlob.BasicStats_swiginit(self, _p3dBlob.new_BasicStats())
    __swig_destroy__ = _p3dBlob.delete_BasicStats

# Register BasicStats in _p3dBlob:
_p3dBlob.BasicStats_swigregister(BasicStats)

class TextureStats(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FD = property(_p3dBlob.TextureStats_FD_get, _p3dBlob.TextureStats_FD_set)

    def __init__(self):
        _p3dBlob.TextureStats_swiginit(self, _p3dBlob.new_TextureStats())
    __swig_destroy__ = _p3dBlob.delete_TextureStats

# Register TextureStats in _p3dBlob:
_p3dBlob.TextureStats_swigregister(TextureStats)


def p3dBlobAnalysis(in_im, out_stats, blob_im, star_im, dimx, dimy, dimz, voxelsize, conn, max_rot, skip_borders, wr_log):
    return _p3dBlob.p3dBlobAnalysis(in_im, out_stats, blob_im, star_im, dimx, dimy, dimz, voxelsize, conn, max_rot, skip_borders, wr_log)

def p3dBasicAnalysis(in_im, out_stats, dimx, dimy, dimz, voxelsize, wr_log):
    return _p3dBlob.p3dBasicAnalysis(in_im, out_stats, dimx, dimy, dimz, voxelsize, wr_log)

def p3dTextureAnalysis(in_im, out_stats, dimx, dimy, dimz, wr_log):
    return _p3dBlob.p3dTextureAnalysis(in_im, out_stats, dimx, dimy, dimz, wr_log)

def p3dAnisotropyAnalysis(in_im, msk_im, out_stats, dimx, dimy, dimz, voxelsize, verbose, wr_log):
    return _p3dBlob.p3dAnisotropyAnalysis(in_im, msk_im, out_stats, dimx, dimy, dimz, voxelsize, verbose, wr_log)

def p3dMorphometricAnalysis(in_im, msk_im, out_stats, dimx, dimy, dimz, voxelsize, wr_log):
    return _p3dBlob.p3dMorphometricAnalysis(in_im, msk_im, out_stats, dimx, dimy, dimz, voxelsize, wr_log)

def p3dREVEstimation(in_rev, porosity, cubeEdges, numel, dimx, dimy, dimz, stepsize, centerx, centery, centerz, wr_log):
    return _p3dBlob.p3dREVEstimation(in_rev, porosity, cubeEdges, numel, dimx, dimy, dimz, stepsize, centerx, centery, centerz, wr_log)

def p3dChamferDT(in_im, out_im, dimx, dimy, dimz, w1, w2, w3, wr_log):
    return _p3dBlob.p3dChamferDT(in_im, out_im, dimx, dimy, dimz, w1, w2, w3, wr_log)

def p3dBlobLabeling_ushort(in_im, out_im, dimx, dimy, dimz, conn, random_lbl, skip_borders, wr_log):
    return _p3dBlob.p3dBlobLabeling_ushort(in_im, out_im, dimx, dimy, dimz, conn, random_lbl, skip_borders, wr_log)

def p3dBlobLabeling_uint(in_im, out_im, dimx, dimy, dimz, conn, random_lbl, skip_borders, wr_log):
    return _p3dBlob.p3dBlobLabeling_uint(in_im, out_im, dimx, dimy, dimz, conn, random_lbl, skip_borders, wr_log)

def p3dGetMaxVolumeBlob3D(in_rev, out_rev, dimx, dimy, dimz, conn, wr_log):
    return _p3dBlob.p3dGetMaxVolumeBlob3D(in_rev, out_rev, dimx, dimy, dimz, conn, wr_log)

def p3dGetMinVolumeBlob3D(in_rev, out_rev, dimx, dimy, dimz, conn, wr_log):
    return _p3dBlob.p3dGetMinVolumeBlob3D(in_rev, out_rev, dimx, dimy, dimz, conn, wr_log)

def p3dMinVolumeFilter3D(in_im, out_im, dimx, dimy, dimz, min_volume, conn, wr_log):
    return _p3dBlob.p3dMinVolumeFilter3D(in_im, out_im, dimx, dimy, dimz, min_volume, conn, wr_log)

def p3dSquaredEuclideanDT(in_rev, out_rev, dimx, dimy, dimz, wr_log):
    return _p3dBlob.p3dSquaredEuclideanDT(in_rev, out_rev, dimx, dimy, dimz, wr_log)

def p3dResetStartTime():
    return _p3dBlob.p3dResetStartTime()

def p3dGetElapsedTime():
    return _p3dBlob.p3dGetElapsedTime()

def p3dGetElapsedTime_min():
    return _p3dBlob.p3dGetElapsedTime_min()

def p3dGetElapsedTime_sec():
    return _p3dBlob.p3dGetElapsedTime_sec()
class bb_lelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    elem = property(_p3dBlob.bb_lelem_t_elem_get, _p3dBlob.bb_lelem_t_elem_set)
    next = property(_p3dBlob.bb_lelem_t_next_get, _p3dBlob.bb_lelem_t_next_set)

    def __init__(self):
        _p3dBlob.bb_lelem_t_swiginit(self, _p3dBlob.new_bb_lelem_t())
    __swig_destroy__ = _p3dBlob.delete_bb_lelem_t

# Register bb_lelem_t in _p3dBlob:
_p3dBlob.bb_lelem_t_swigregister(bb_lelem_t)


def bb_list_init(list):
    return _p3dBlob.bb_list_init(list)

def bb_list_add(list, item):
    return _p3dBlob.bb_list_add(list, item)

def bb_list_isempty(list):
    return _p3dBlob.bb_list_isempty(list)

def bb_list_toarray(list, numel):
    return _p3dBlob.bb_list_toarray(list, numel)

def bb_list_clear(list):
    return _p3dBlob.bb_list_clear(list)
class bb_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    min_x = property(_p3dBlob.bb_t_min_x_get, _p3dBlob.bb_t_min_x_set)
    max_x = property(_p3dBlob.bb_t_max_x_get, _p3dBlob.bb_t_max_x_set)
    min_y = property(_p3dBlob.bb_t_min_y_get, _p3dBlob.bb_t_min_y_set)
    max_y = property(_p3dBlob.bb_t_max_y_get, _p3dBlob.bb_t_max_y_set)
    min_z = property(_p3dBlob.bb_t_min_z_get, _p3dBlob.bb_t_min_z_set)
    max_z = property(_p3dBlob.bb_t_max_z_get, _p3dBlob.bb_t_max_z_set)

    def __init__(self):
        _p3dBlob.bb_t_swiginit(self, _p3dBlob.new_bb_t())
    __swig_destroy__ = _p3dBlob.delete_bb_t

# Register bb_t in _p3dBlob:
_p3dBlob.bb_t_swigregister(bb_t)


def p3dConnectedComponentsLabeling_ushort(in_rev, out_rev, numOfConnectedComponents, volumes, boundingBoxes, dimx, dimy, dimz, conn, random_lbl, skip_borders):
    return _p3dBlob.p3dConnectedComponentsLabeling_ushort(in_rev, out_rev, numOfConnectedComponents, volumes, boundingBoxes, dimx, dimy, dimz, conn, random_lbl, skip_borders)

def p3dConnectedComponentsLabeling_uint(in_rev, out_rev, numOfConnectedComponents, volumes, boundingBoxes, dimx, dimy, dimz, conn, random_lbl, skip_borders):
    return _p3dBlob.p3dConnectedComponentsLabeling_uint(in_rev, out_rev, numOfConnectedComponents, volumes, boundingBoxes, dimx, dimy, dimz, conn, random_lbl, skip_borders)
class coords_lelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    elem = property(_p3dBlob.coords_lelem_t_elem_get, _p3dBlob.coords_lelem_t_elem_set)
    next = property(_p3dBlob.coords_lelem_t_next_get, _p3dBlob.coords_lelem_t_next_set)

    def __init__(self):
        _p3dBlob.coords_lelem_t_swiginit(self, _p3dBlob.new_coords_lelem_t())
    __swig_destroy__ = _p3dBlob.delete_coords_lelem_t

# Register coords_lelem_t in _p3dBlob:
_p3dBlob.coords_lelem_t_swigregister(coords_lelem_t)


def coords_list_init(arg1):
    return _p3dBlob.coords_list_init(arg1)

def coords_list_push(arg1, arg2):
    return _p3dBlob.coords_list_push(arg1, arg2)

def coords_list_pop(arg1):
    return _p3dBlob.coords_list_pop(arg1)

def coords_list_isempty(arg1):
    return _p3dBlob.coords_list_isempty(arg1)

def coords_list_toarray(arg1, arg2):
    return _p3dBlob.coords_list_toarray(arg1, arg2)
class coords_qelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    item = property(_p3dBlob.coords_qelem_t_item_get, _p3dBlob.coords_qelem_t_item_set)
    next = property(_p3dBlob.coords_qelem_t_next_get, _p3dBlob.coords_qelem_t_next_set)

    def __init__(self):
        _p3dBlob.coords_qelem_t_swiginit(self, _p3dBlob.new_coords_qelem_t())
    __swig_destroy__ = _p3dBlob.delete_coords_qelem_t

# Register coords_qelem_t in _p3dBlob:
_p3dBlob.coords_qelem_t_swigregister(coords_qelem_t)

class coords_queue_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    tail = property(_p3dBlob.coords_queue_t_tail_get, _p3dBlob.coords_queue_t_tail_set)
    head = property(_p3dBlob.coords_queue_t_head_get, _p3dBlob.coords_queue_t_head_set)

    def __init__(self):
        _p3dBlob.coords_queue_t_swiginit(self, _p3dBlob.new_coords_queue_t())
    __swig_destroy__ = _p3dBlob.delete_coords_queue_t

# Register coords_queue_t in _p3dBlob:
_p3dBlob.coords_queue_t_swigregister(coords_queue_t)


def coords_queue_init(queue):
    return _p3dBlob.coords_queue_init(queue)

def coords_queue_push(queue, elem):
    return _p3dBlob.coords_queue_push(queue, elem)

def coords_queue_pop(queue):
    return _p3dBlob.coords_queue_pop(queue)

def coords_queue_isempty(queue):
    return _p3dBlob.coords_queue_isempty(queue)
class coords_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_p3dBlob.coords_t_x_get, _p3dBlob.coords_t_x_set)
    y = property(_p3dBlob.coords_t_y_get, _p3dBlob.coords_t_y_set)
    z = property(_p3dBlob.coords_t_z_get, _p3dBlob.coords_t_z_set)

    def __init__(self):
        _p3dBlob.coords_t_swiginit(self, _p3dBlob.new_coords_t())
    __swig_destroy__ = _p3dBlob.delete_coords_t

# Register coords_t in _p3dBlob:
_p3dBlob.coords_t_swigregister(coords_t)

class fcoords_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_p3dBlob.fcoords_t_x_get, _p3dBlob.fcoords_t_x_set)
    y = property(_p3dBlob.fcoords_t_y_get, _p3dBlob.fcoords_t_y_set)
    z = property(_p3dBlob.fcoords_t_z_get, _p3dBlob.fcoords_t_z_set)

    def __init__(self):
        _p3dBlob.fcoords_t_swiginit(self, _p3dBlob.new_fcoords_t())
    __swig_destroy__ = _p3dBlob.delete_fcoords_t

# Register fcoords_t in _p3dBlob:
_p3dBlob.fcoords_t_swigregister(fcoords_t)

class double_lelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ct = property(_p3dBlob.double_lelem_t_ct_get, _p3dBlob.double_lelem_t_ct_set)
    next = property(_p3dBlob.double_lelem_t_next_get, _p3dBlob.double_lelem_t_next_set)

    def __init__(self):
        _p3dBlob.double_lelem_t_swiginit(self, _p3dBlob.new_double_lelem_t())
    __swig_destroy__ = _p3dBlob.delete_double_lelem_t

# Register double_lelem_t in _p3dBlob:
_p3dBlob.double_lelem_t_swigregister(double_lelem_t)


def double_list_init(list):
    return _p3dBlob.double_list_init(list)

def double_list_add(list, item):
    return _p3dBlob.double_list_add(list, item)

def double_list_isempty(list):
    return _p3dBlob.double_list_isempty(list)

def double_list_toarray(list, numel):
    return _p3dBlob.double_list_toarray(list, numel)
class fcoords_lelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    elem = property(_p3dBlob.fcoords_lelem_t_elem_get, _p3dBlob.fcoords_lelem_t_elem_set)
    next = property(_p3dBlob.fcoords_lelem_t_next_get, _p3dBlob.fcoords_lelem_t_next_set)

    def __init__(self):
        _p3dBlob.fcoords_lelem_t_swiginit(self, _p3dBlob.new_fcoords_lelem_t())
    __swig_destroy__ = _p3dBlob.delete_fcoords_lelem_t

# Register fcoords_lelem_t in _p3dBlob:
_p3dBlob.fcoords_lelem_t_swigregister(fcoords_lelem_t)


def fcoords_list_init(arg1):
    return _p3dBlob.fcoords_list_init(arg1)

def fcoords_list_push(arg1, arg2):
    return _p3dBlob.fcoords_list_push(arg1, arg2)

def fcoords_list_pop(arg1):
    return _p3dBlob.fcoords_list_pop(arg1)

def fcoords_list_isempty(arg1):
    return _p3dBlob.fcoords_list_isempty(arg1)

def fcoords_list_toarray(arg1, arg2):
    return _p3dBlob.fcoords_list_toarray(arg1, arg2)
class uint_lelem_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ct = property(_p3dBlob.uint_lelem_t_ct_get, _p3dBlob.uint_lelem_t_ct_set)
    next = property(_p3dBlob.uint_lelem_t_next_get, _p3dBlob.uint_lelem_t_next_set)

    def __init__(self):
        _p3dBlob.uint_lelem_t_swiginit(self, _p3dBlob.new_uint_lelem_t())
    __swig_destroy__ = _p3dBlob.delete_uint_lelem_t

# Register uint_lelem_t in _p3dBlob:
_p3dBlob.uint_lelem_t_swigregister(uint_lelem_t)


def uint_list_init(list):
    return _p3dBlob.uint_list_init(list)

def uint_list_add(list, ct):
    return _p3dBlob.uint_list_add(list, ct)

def uint_list_isempty(list):
    return _p3dBlob.uint_list_isempty(list)

def uint_list_toarray(list, numel):
    return _p3dBlob.uint_list_toarray(list, numel)

def uint_list_clear(list):
    return _p3dBlob.uint_list_clear(list)

def findNeighbor(im, dimx, dimy, dimz, i, j, k, coords):
    return _p3dBlob.findNeighbor(im, dimx, dimy, dimz, i, j, k, coords)

def countNeighbors(im, dimx, dimy, dimz, i, j, k):
    return _p3dBlob.countNeighbors(im, dimx, dimy, dimz, i, j, k)

def isSimplePoint(im, dimx, dimy, dimz, x, y, z):
    return _p3dBlob.isSimplePoint(im, dimx, dimy, dimz, x, y, z)

def p3dCrop3D_uchar2uchar(in_im, out_im, dimx, dimy, dimz, size):
    return _p3dBlob.p3dCrop3D_uchar2uchar(in_im, out_im, dimx, dimy, dimz, size)

def p3dCrop3D_ushort2ushort(in_im, out_im, dimx, dimy, dimz, size):
    return _p3dBlob.p3dCrop3D_ushort2ushort(in_im, out_im, dimx, dimy, dimz, size)

def p3dCrop3D_uint2uint(in_rev, out_rev, dimx, dimy, dimz, size):
    return _p3dBlob.p3dCrop3D_uint2uint(in_rev, out_rev, dimx, dimy, dimz, size)

def p3dCrop3D_float2float(in_im, out_im, dimx, dimy, dimz, size):
    return _p3dBlob.p3dCrop3D_float2float(in_im, out_im, dimx, dimy, dimz, size)

def p3dZeroPadding3D_uchar2float(in_im, out_im, dimx, dimy, dimz, size):
    return _p3dBlob.p3dZeroPadding3D_uchar2float(in_im, out_im, dimx, dimy, dimz, size)

def p3dZeroPadding3D_uchar2uchar(in_im, out_im, dimx, dimy, dimz, size):
    return _p3dBlob.p3dZeroPadding3D_uchar2uchar(in_im, out_im, dimx, dimy, dimz, size)

def p3dReplicatePadding3D_uchar2uchar(in_im, out_im, dimx, dimy, dimz, size):
    return _p3dBlob.p3dReplicatePadding3D_uchar2uchar(in_im, out_im, dimx, dimy, dimz, size)

def interpolation(gvf, dimx, dimy, dimz, x, y, z):
    return _p3dBlob.interpolation(gvf, dimx, dimy, dimz, x, y, z)

def isBoundary(in_im, dimx, dimy, dimz, i, j, k):
    return _p3dBlob.isBoundary(in_im, dimx, dimy, dimz, i, j, k)

def isFullNeighborhood(in_im, dimx, dimy, dimz, i, j, k):
    return _p3dBlob.isFullNeighborhood(in_im, dimx, dimy, dimz, i, j, k)

def p3dSpecialPadding3D_uchar2uchar(in_im, out_im, dimx, dimy, dimz, size):
    return _p3dBlob.p3dSpecialPadding3D_uchar2uchar(in_im, out_im, dimx, dimy, dimz, size)





import p3dBlob
from p3dBlob import *


################## constants
P3D_FALSE=-1 
P3D_TRUE=1 

P3D_ERROR=0
P3D_MEM_ERROR=None
P3D_SUCCESS=2

BACKGROUND=0
#OBJECT=UCHAR_MAX

CONN4=611
CONN8=612

CONN6=711
CONN18=712
CONN26=713

################## error messages

def py_printErrorMessage(err_code):
  
    string = "Success. \n"
  
    if err_code == 1 or err_code == 0:
        string = "Error on code execution. \n"
        return
  	
    if err_code == -2:
        string = "Input argument IMAGE must be of type BYTE or UINT. \n"
        return
  	
    if err_code == -3:
        string = "Input argument IMAGE must be a 2D or 3D matrix. \n"
        return
   	
    if err_code == 2:
        print (string)

#################
  
#Basic analysis
def py_p3dBasicAnalysis(image_data, dimx,dimy,dimz, resolution = 1.0):
    out_BasicStats = BasicStats()
    err_code = p3dBasicAnalysis(image_data,out_BasicStats,dimx,dimy,dimz, resolution, None)
    py_printErrorMessage(err_code)
    return out_BasicStats

#Anisotrtopy analysis
def py_p3dAnisotropyAnalysis(image_data, dimx,dimy,dimz, resolution = 1.0, details = False):
    out_AnisotropyStats = AnisotropyStats()
    im_dummy_mask = malloc_uchar(dimx*dimy*dimz)
    im_dummy_mask = image_data
    err_code = p3dAnisotropyAnalysis(image_data, None, out_AnisotropyStats, dimx,dimy,dimz,resolution,details , None)
    py_printErrorMessage(err_code)
    return out_AnisotropyStats

#Blob labeling
def py_p3dBlobLabeling_8(image_data, dimx,dimy,dimz, conn3D = CONN6 , rand = 0, skip_borders = 0):
    out_image = malloc_ushort(dimx*dimy*dimz)
    err_code = p3dBlobLabeling_uint(image_data, out_image, dimx,dimy,dimz,conn3D,rand , skip_borders, None)
    py_printErrorMessage(err_code)
    return out_image

#Blob Get max
def py_p3dGetMaxVolumeBlob3D(image_data, dimx,dimy,dimz, conn3D = CONN6):
    out_image = malloc_uchar(dimx*dimy*dimz)
    err_code = p3dGetMaxVolumeBlob3D(image_data, out_image, dimx,dimy,dimz,conn3D, None)
    py_printErrorMessage(err_code)
    return out_image

#Blob Get min
def py_p3dGetMinVolumeBlob3D(image_data, dimx,dimy,dimz, conn3D = CONN6):
    out_image = malloc_uchar(dimx*dimy*dimz)
    err_code = p3dGetMinVolumeBlob3D(image_data, out_image, dimx,dimy,dimz,conn3D, None)
    py_printErrorMessage(err_code)
    return out_image

#ChamferDT
def py_p3dChamferDT(image_data, dimx,dimy,dimz, w1 = 3, w2 = 4, w3 = 5):
    out_image = malloc_ushort(dimx*dimy*dimz)
    err_code = p3dChamferDT(image_data, out_image, dimx,dimy,dimz,w1, w2, w3, None)
    py_printErrorMessage(err_code)
    return out_image

#SquaredEuclideanDT
def py_p3dSquaredEuclideanDT(image_data, dimx,dimy,dimz):
    out_image = malloc_uchar(dimx*dimy*dimz)
    err_code = p3dSquaredEuclideanDT(image_data, out_image, dimx,dimy,dimz, None)
    py_printErrorMessage(err_code)
    return out_image

# MinVolumeFilter
def py_p3dMinVolumeFilter3D(image_data, dimx,dimy,dimz,min_vol = 5):
    out_image = malloc_uchar(dimx*dimy*dimz)
    err_code = p3dMinVolumeFilter3D(image_data, out_image, dimx,dimy,dimz, min_vol, None)
    py_printErrorMessage(err_code)
    return out_image

# Morphometric Analysis
def py_p3dMorphometricAnalysis(image_data, mask_image, dimx,dimy,dimz,resolution = 1.0):
    out_MorphometricStat = MorphometricStats()
    mask_image = malloc_uchar(dimx*dimy*dimz)
    err_code = p3dMorphometricAnalysis(image_data, mask_image, out_MorphometricStat, dimx,dimy,dimz, resolution, None)
    py_printErrorMessage(err_code)
    return out_MorphometricStat

# Texture Analysis
def py_p3dTextureAnalysis(image_data, dimx,dimy,dimz):
    out_TextureStat = TextureStats()
    err_code = p3dTextureAnalysis(image_data, out_MorphometricStat, dimx,dimy,dimz, None)
    py_printErrorMessage(err_code)
    return out_TextureStat

# Blob Analysis
def py_p3dBlobAnalysis(image_data,dimx,dimy,dimz, blob_im = None, star_im = None, resolution = 1.0, conn = 6, max_rot = 1024, skip_borders = 0):
    out_BlobStat = BlobStats()
    if conn == 6 :
        conn = CONN6;
    if conn == 18 :
        conn = CONN18;
    if conn == 26 :
        conn = CONN26;
        
    if skip_borders == 0 :
        borders = P3D_FALSE;
    else :
        borders = P3D_TRUE;
        
    err_code = p3dBlobAnalysis(image_data, out_BlobStat, blob_im, star_im, dimx,dimy,dimz,resolution,conn,max_rot,borders, None)
    py_printErrorMessage(err_code)
    return out_BlobStat

#IDL_VPTR p3d_idlREVEstimation(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlBlobAnalysis(int, IDL_VPTR*, char* );



import p3dSkel
from p3dSkel import *

################## 

def py_printErrorMessage(err_code):
  
  string = "Success. \n"
  
  if err_code == 1 or err_code == 0:
  	string = "Error on code execution. \n"
  	return
  	
  if err_code == -2:
   	string = "Input argument IMAGE must be of type BYTE or UINT. \n"
   	return
  	
  if err_code == -3:
   	string = "Input argument IMAGE must be a 2D or 3D matrix. \n"
   	return
   	
  if err_code == 2:
  	print (string)

  
################## Handle 8 bit
def py_p3dSkeletonLabeling(image_data, dimx,dimy,dimz):
    out_image = malloc_uchar(dimx*dimy*dimz)
    err_code = p3dSkeletonLabeling(image_data,out_image,dimx,dimy,dimz, None)
    py_printErrorMessage(err_code)
    return out_image
   
def py_p3dGVFSkeletonization(image_data, dimx,dimy,dimz,mu= 0.15,eps= 1E-4,hierarc = 1.0,scale= 0.0):
    out_image = malloc_uchar(dimx*dimy*dimz)
    err_code = p3dGVFSkeletonization(image_data,out_image,dimx,dimy,dimz, mu,ep,hierarc,scale, None)
    py_printErrorMessage(err_code)
    return out_image

def py_p3dLKCSkeletonization(image_data, dimx,dimy,dimz):
    out_image = P3D_Skel_Py.p3dSkel.malloc_uchar(dimx*dimy*dimz)
    err_code = p3dLKCSkeletonization(image_data,out_image,dimx,dimy,dimz, None)
    py_printErrorMessage(err_code)
    return out_image

def py_p3d_p3dThinningSkeletonization(image_data, dimx,dimy,dimz):
    out_image = malloc_uchar(dimx*dimy*dimz)
    err_code = p3dThinningSkeletonization(image_data,out_image,dimx,dimy,dimz, None)
    py_printErrorMessage(err_code)
    return out_image

def py_p3dSkeletonAnalysis(image_data,skeleton_image,skeleton_stats,dimx,dimy,dimz,nodes_im = None, pores_im= None,ends_im= None,throats_im= None,merging_factor= 0.85, tortuosity_depth= 3, resolution= 1.0):
    out_image = malloc_uchar(dimx*dimy*dimz)
    err_code=p3dSkeletonAnalysis(image_data,skeleton_image,skeleton_stats,nodes_im,pores_im,ends_im,throats_im,dimx,dimy,dimz,merging_factor,tortuosity_depth, resolution,None)
    py_printErrorMessage(err_code)
    return out_image

    
def py_p3dSkeletonPruning(image_data,dimx,dimy,dimz,thresh=3,ultimate = False,iterative = False):
    out_image = malloc_uchar(dimx*dimy*dimz)
    if iterative == True and ultimate == False:
        print("p3dIterativeSkeletonPruning")
        err_code = p3dIterativeSkeletonPruning (image_data, out_image,dimx,dimy,dimz, thresh, None )
        py_printErrorMessage(err_code)
        return out_image
    if ultimate == True:
        print("p3dUltimateSkeletonPruning")
        err_code =  p3dUltimateSkeletonPruning (image_data, out_image, dimx,dimy,dimz, iterative, None )
        py_printErrorMessage(err_code)
        return out_image
    # Default: iterative == False and ultimate = False
    print("p3dSimpleSkeletonPruning")
    err_code = p3dSimpleSkeletonPruning(image_data, out_image,dimx,dimy,dimz, thresh, None )
    py_printErrorMessage(err_code)
    return out_image
    

#IDL_VPTR p3d_idlSkeletonAnalysisFeasibility (int, IDL_VPTR*, char* );

	


################## Handle 16 bit
				
#IDL_VPTR p3d_idlPKSkeletonization(int, IDL_VPTR*, char* );
#IDL_VPTR p3d_idlSkeletonAnalysisFeasibility (int, IDL_VPTR*, char* );

  
  
